package cz.muni.fi.pv168.graves;

import java.util.Collection;
import java.util.HashSet;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

/**
 * This is a test for cemetery manager.
 * There are implemented two testing methods.
 *
 * In task 2 of PV168 should be at least 3 of them.
 * 
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 */
public class CemeteryManagerTest {

    private CemeteryManager manager;

    public CemeteryManagerTest() {
    }

    @Before
    public void setUp() {
        manager = new CemeteryManagerImpl();
    }

    /**
     * We wan't to have the database clean after executing each test.
     */
    @After
    public void tearDown() {
        Collection<Grave> graves = manager.getAllGraves();
        for (Grave grave : graves) {
            manager.deleteGrave(grave);
        }

    }

    /**
     * Test of getAllGraves method, of class CemeteryManager.
     */
    @Test
    public void testGetAllGraves() {
        fail("Test not implemented");


    }

    /**
     * Test of getGraveById method, of class CemeteryManager.
     */
    @Test
    public void testGetGraveById() {
        fail("Test not implemented");


    }

    /**
     * Test of addGrave method, of class CemeteryManager.
     * Tests correct adding
     */
    @Test
    public void testAddGrave() {
        Grave grave = createSampleGrave();
        //we try to add a correct grave
        manager.addGrave(grave);
        Long graveId = grave.getId();
        //now id should not be null
        assertNotNull(graveId);
        //we try to get the object back
        Grave result = manager.getGraveById(graveId);
        //should be the same
        assertEquals(grave, result);
        //all the attributes should be the same
        assertEquals(grave.getCapacity(), result.getCapacity());
        assertEquals(grave.getRow(), result.getRow());
        assertEquals(grave.getColumn(), result.getColumn());
        assertEquals(1, manager.getAllGraves().size());

        //we try to add a grave with capacity 1
        //we try to add a correct grave
        grave.setCapacity(1);
        grave.setId(null);
        manager.addGrave(grave);
        graveId = grave.getId();
        result = manager.getGraveById(graveId);
        //should be the same
        assertEquals(grave, result);
        //all the attributes should be the same
        assertEquals(grave.getCapacity(), result.getCapacity());
        assertEquals(grave.getRow(), result.getRow());
        assertEquals(grave.getColumn(), result.getColumn());
        assertEquals(2, manager.getAllGraves().size());
    }

    /**
     * Test of addGrave method, of class CemeteryManager.
     * Tests incorrect adding
     */
    @Test
    public void testAddGraveErrors() {
        Grave grave = createSampleGrave();
        //when we try to add null, we should get an exception
        try {
            manager.addGrave(null);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
        grave.setId(5l);
        //when we try to add a grave with set id, we should get an exception
        try {
            manager.addGrave(grave);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

        //when we try to add a grave with negative capacity, we should get an exception
        grave.setId(null);
        grave.setCapacity(-1);
        try {
            manager.addGrave(grave);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }

        //when we try to add a grave with zero capacity, we should get an exception
        grave.setId(null);
        grave.setCapacity(0);
        try {
            manager.addGrave(grave);
            fail();
        } catch (IllegalArgumentException ex) {
            //OK
        }
    }

    /**
     * Test of deleteGrave method, of class CemeteryManager.
     */
    @Test
    public void testDeleteGrave() {
        fail("Test not implemented");
    }

    /**
     * Test of updateGrave method, of class CemeteryManager.
     */
    @Test
    public void testUpdateGrave() {
        fail("test not implemented");
    }

    private Grave createSampleGrave() {
        Grave grave = new Grave();
        grave.setCapacity(4);
        grave.setBodies(new HashSet<Body>());
        grave.setColumn(1);
        grave.setRow(1);
        return grave;
    }
}
